// Copyright Maarten L. Hekkelman, Radboud University 2008-2013.
//        Copyright Maarten L. Hekkelman, 2014-2023
//  Distributed under the Boost Software License, Version 1.0.
//     (See accompanying file LICENSE_1_0.txt or copy at
//           http://www.boost.org/LICENSE_1_0.txt)

/// \file
/// Generic configuration file, contains defines and (probably obsolete) stuff for msvc

#pragma once

/// The http server implementation in libzeep can use a
/// preforked mode. That means the main process listens to
/// a network port and passes the socket to a client process
/// for doing the actual handling. The advantages for a setup
/// like this is that if the client fails, the server can detect
/// this and restart the client thereby guaranteeing a better
/// uptime.

#ifndef HTTP_SERVER_HAS_PREFORK
#define HTTP_SERVER_HAS_PREFORK 1
#endif

/// The http::daemon class can use fork/exec to spawn a
/// process in the background. That's not going to work
/// on Windows for now.

#ifndef HTTP_HAS_UNIX_DAEMON
#define HTTP_HAS_UNIX_DAEMON 1
#endif

/// The webapp class in libzeep can use resources to load files.
/// In this case you need the resource compiler 'mrc' obtainable
/// from https://github.com/mhekkel/mrc

#ifndef WEBAPP_USES_RESOURCES
#define WEBAPP_USES_RESOURCES 1
#endif

/// We can use resources to store the contents of a docroot e.g.
/// which makes web applications really portable. However, for
/// this feature we need a working mrc
/// (https://github.com/mhekkel/mrc) and that is limited to
/// Windows and operating systems using the ELF executable format.

#ifndef USE_RSRC
#define USE_RSRC 1
#endif

/// The current version of libzeep

#define LIBZEEP_VERSION "6.1.0"
#define LIBZEEP_VERSION_MAJOR 6
#define LIBZEEP_VERSION_MINOR 1
#define LIBZEEP_VERSION_PATCH 0

